/*
 * Copyright 2007 Evan Cowden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.nakedosgi.annotations.scr;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import static org.nakedosgi.Constants.*;

/**
 * Annotation for defining a service reference, as per the OSGi Declaritive
 * Services specification.  In particular, this annotation is used to 
 * declare references that bind using the event-based model.  For other 
 * references, use the {@link SCRReference} annotation.
 * Evan Cowden
 */
@Documented
@Inherited
@Retention(RetentionPolicy.SOURCE)
@Target(ElementType.METHOD)
public @interface SCRBind {
    
    /**
     * The name of the property reference.  Make 
     * sure that the name used in the {@code SCRBind} annotation matches that
     * used in the corresponding {@code Unbind} annotation.
     */
    String name() default NO_VALUE;

    /**
     * <p>If no name is specified, defaults to the type of the property.</p>
     * <p>{@code _interface} is used as opposed to <i>interface</i> since
     * <i>interface</i> is a reserved word in Java.</p>
     */
    Class<?> interface_() default org.nakedosgi.annotations.NoValue.class;

    /**
     * If no name is specified, defaults to {@code Cardinality.EXACTLY_ONE}
     */
    Cardinality cardinality() default Cardinality.EXACTLY_ONE;
    
    /**
     * If no name is specified, defaults to {@code Policy.STATIC}
     */
    Policy policy() default Policy.STATIC;
    
    /**
     * If no name is specified, no {@code target} attribute will be included
     * in the service reference definition. 
     */
    String target() default NO_VALUE;
}
