/*
 * Copyright 2008 Evan Cowden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.nakedosgi.annotations.scr;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import static org.nakedosgi.Constants.*;

/**
 * <p>SCR Reference configuration.  This annotation can only fit within a 
 * {@link SCRComponent} annotation Note that this annotation
 * <i>CANNOT</i> be applied anywhere outside of this annotation.</p>
 * 
 * <p>Component references that use the event-based binding model
 * should generally be configured with the {@link SCRBind} and {@link SCRUnbind} 
 * annotations.</p>
 * @author Evan Cowden
 */
@Documented
@Retention(value = RetentionPolicy.SOURCE)
@Target(value = ElementType.ANNOTATION_TYPE)  //TODO not really right...
public @interface SCRReference {
    
    String name();
    
    Class<?> interfaceClass();
    
    Cardinality cardinality() default Cardinality.EXACTLY_ONE;
    
    Policy policy() default Policy.STATIC;
    
    String target() default NO_VALUE;
    
    String bind() default NO_VALUE;
    
    String unbind() default NO_VALUE;
}
