/*
 * Copyright 2007 Evan Cowden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.nakedosgi.manifest;

import java.util.Map;

/**
 * Utility methods for this package
 * @author Evan Cowden
 */
class Util {

    private static final String DELIMITER = ", ";

    public static void addIfNewKey(Map<String, String> map, String key, String value) {
        if (!map.containsKey(key)) {
            map.put(key, value);
        }
    }

    /**
     * Append the specified value to the map in a comma-delimited list, or create
     * the key with only one value if none exists.
     * @param map
     * @param key
     * @param value
     */
    public static void appendValue(Map<String, String> map, String key, String value) {
        //TODO check for duplicates
        if (map.containsKey(key)) {
            StringBuilder builder = new StringBuilder();
            builder.append(map.get(key));
            builder.append(DELIMITER);
            builder.append(value);
            map.put(key, builder.toString());
        } else {
            map.put(key, value);
        }
    }
}
