/*
 * Copyright 2008 Evan Cowden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.nakedosgi.processor.scr;

import java.beans.Introspector;
import java.util.Collection;
import javax.lang.model.element.ExecutableElement;

import org.nakedosgi.processor.model.Component;
import org.nakedosgi.processor.model.Reference;

import static org.nakedosgi.Constants.*;

/**
 * Utility methods for processing {@link SCRBind} and {@SCRUnbind} annotations
 * @author Evan Cowden
 */
public class BindUnbindUtil {
    /**
     * Check that the parameters are correct
     * @param element
     * @return
     */
    public static boolean parametersAreInvalid(ExecutableElement element) {
        if (element.getParameters().size() != 1) {
            return true;
        }
        return false;
    }

    /**
     * Get the reference name given the specified element and annotation
     * @param element
     * @param annotation
     * @return
     */
    public static String getName(ExecutableElement element, String annotationName) {
        //...name
        String name;
        if (annotationName.equals(NO_VALUE)) {
            String methodName = element.getSimpleName().toString();
            name = stripFirstWord(methodName);
        } else {
            name = annotationName;
        }

        return name;
    }

    /**
     * Get a {@link Reference} of the specified name from the specified component.
     * If it does not exist, create it and add it to the component.
     * @param name
     * @param component
     * @return
     */
    public static Reference getReference(String name, Component component) {
        Collection<Reference> references = component.getReference();

        for (Reference reference : references) {
            if (reference.getName().equals(name)) {
                return reference;
            }
        }

        Reference result = new Reference();
        result.setName(name);
        references.add(result);

        return result;
    }

    /**
     * Strip the first word from a string, using standard camel-case.  
     * For instance, "getName" becomes "name."  If there are no capitals, the
     * string is returned decapitalized but not stripped.
     * @param string
     * @return
     */
    public static String stripFirstWord(String string) {
        for (int i = 1; i < string.length(); i++) {
            if (Character.isUpperCase(string.charAt(i))) {
                String result = string.substring(i);
                return Introspector.decapitalize(result);
            }
        }

        //No capitals found
        return Introspector.decapitalize(string);
    }
}
